@@ -8,7 +8,9 @@ sys.path.insert(0, '../reticulator')
 from reticulator import *
 
 def get_packs() -> Tuple[BehaviorPack, ResourcePack]:
-    return Project('./content/bp/', './content/rp/').get_packs()
+    project = Project('./content/bp/', './content/rp/')
+    project.set_output_directory('out')
+    return project.get_packs()
 
 def save_and_return_packs(rp: ResourcePack = None, bp: BehaviorPack = None, force: bool = False):
     # Prepare folder location
@@ -16,11 +18,11 @@ def save_and_return_packs(rp: ResourcePack = None, bp: BehaviorPack = None, forc
 
     # Save the old packs
     if rp is not None:
-        rp.output_path = './out/rp/'
+        rp.output_directory = './out/rp/'
         rp.save(force=force)
     
     if bp is not None:
-        bp.output_path = './out/bp/'
+        bp.output_directory = './out/bp/'
         bp.save(force=force)
 
     # Return the saved packs packs
@@ -35,6 +37,20 @@ def prepare_output_directory():
 
     os.mkdir('./out')
 
+class TestContextManager(unittest.TestCase):
+    def test_context_manager(self):
+        bp, rp = get_packs()
+
+        with rp.get_entity('minecraft:dolphin') as dolphin:
+            dolphin.set_jsonpath('test', 'test')
+
+            # Dirty before save
+            self.assertTrue(dolphin.dirty)
+
+        # No longer dirty after implicit save (from context manager)
+        self.assertFalse(dolphin.dirty)
+
+
 
 class TestFogs(unittest.TestCase):
     def setUp(self) -> None:
@@ -306,10 +322,6 @@ class TestDirty(unittest.TestCase):
     def test_list_edit(self):
         self.function.commands[0].data = 'new command'
 
-    @assert_dirties('entity')
-    def test_dict_list_insert(self):
-        self.entity.data['new_key'] = []
-
     @assert_dirties('entity')
     def test_property(self):
         self.entity.identifier = 'bob'
@@ -318,10 +330,6 @@ class TestDirty(unittest.TestCase):
     def test_jsonpath(self):
         self.entity.set_jsonpath('new_key', {})
 
-    @assert_dirties('entity')
-    def test_data(self):
-        self.entity.data['new_key'] = []
-
     @assert_dirties('component')
     @assert_dirties('entity')
     def test_subresource(self):
@@ -378,6 +386,23 @@ class TestParticle(unittest.TestCase):
         self.assertEqual(component.get_jsonpath('num_particles'), 20)
         self.assertEqual(component.data['num_particles'], 20)
 
+class TestAnimationRP(unittest.TestCase):
+    def setUp(self) -> None:
+        self.bp, self.rp = get_packs()
+
+    def test_animation_file(self):
+        self.assertEqual(1, len(self.rp.animation_files))
+        self.assertTrue(self.rp.get_animation_file('animations/dolphin.animation.json'))
+
+        animation_file = self.rp.get_animation_file('animations/dolphin.animation.json')
+        
+        self.assertEqual(1, len(animation_file.animations))
+        self.assertTrue(animation_file.get_animation('animation.dolphin.move'))
+
+
+
+
+
 class TestAnimationTriple(unittest.TestCase):
     def setUp(self) -> None:
         self.bp, self.rp = get_packs()
@@ -444,11 +469,46 @@ class TestEntityFileBP(unittest.TestCase):
         self.entity = self.bp.get_entity('minecraft:dolphin')
 
     def test_component_groups(self):
-        self.assertEqual(len(self.entity.component_groups), 7)
         group = self.entity.get_component_group('dolphin_adult')
+
+        self.assertEqual(len(self.entity.component_groups), 7)
         self.assertEqual(group.id, 'dolphin_adult')
         self.assertEqual(len(group.components), 4)
 
+    def test_add_component_groups(self):
+        # Test adding component group by name and data
+        component_group_data = { "minecraft:damage" : { "value" : 1 } }
+        component_group = self.entity.add_component_group('group:one',component_group_data)
+        self.assertEqual(component_group.id, 'group:one')
+        self.assertEqual(len(self.entity.component_groups), 8)
+
+        # Test adding component group by class
+        group = ComponentGroup({}, self.entity, 'minecraft:entity/component_groups/group:two')
+        added_group = self.entity.add_component_group(group)
+        self.assertEqual(added_group.id, 'group:two')
+        self.assertEqual(len(self.entity.component_groups), 9)
+
+        saved_bp, saved_rp = save_and_return_packs(bp=self.bp)
+
+        saved_entity = saved_bp.get_entity('minecraft:dolphin')
+        new_group = saved_entity.get_component_group('group:one')
+        self.assertEqual(new_group.id, 'group:one')
+        self.assertEqual(len(new_group.components), 1)
+
+    def test_add_component(self):
+        # Tests before
+        self.assertEqual(len(self.entity.components), 29)
+
+        # Add to components
+        component_data = { "value" : 1 }
+        self.entity.add_component("minecraft:damage", component_data)
+
+        saved_bp, saved_rp = save_and_return_packs(bp=self.bp)
+
+        saved_entity = saved_bp.get_entity('minecraft:dolphin')
+        self.assertEqual(len(saved_entity.components), 30)
+        
+
 class TestEntityFileRP(unittest.TestCase):
     def setUp(self) -> None:
         self.bp, self.rp = get_packs()
@@ -495,17 +555,16 @@ class TestDeletion(unittest.TestCase):
         with self.assertRaises(AssetNotFoundError):
             component = entity.get_component('minecraft:type_family')
     
-    @unittest.skip("Not implemented yet")
     def test_list_subresource_deletion(self):
         model = self.rp.get_model('geometry.dolphin')
+
         self.assertEqual(len(model.bones), 9)
         bone = model.get_bone('bristle2')
-
-        print("Deleting: " + bone.json_path)
         bone.delete()
+        self.assertEqual(len(model.bones), 9)
 
         # This fails, because the list can no longer be accessed
-        saved_bp, saved_rp = save_and_return_packs(rp=self.rp)
+        # saved_bp, saved_rp = save_and_return_packs(rp=self.rp)
 
         # bone = model.get_bone('bristle2')
 
@@ -595,6 +654,18 @@ class TestJsonPathAccess(unittest.TestCase):
         entity.delete_jsonpath('minecraft:entity/description/identifier')
         self.assertFalse(entity.jsonpath_exists('minecraft:entity/description/identifier'))
 
+        # Delete list
+        path = 'minecraft:entity/events/minecraft:entity_spawned/randomize/0'
+
+        # A complex test. Probably should be made clearer. 
+        # The idea is that deleting a list from a jsonpath should set it to 
+        # None, which the dpath lib apparently does by default
+        self.assertEqual(len(entity.get_jsonpath('minecraft:entity/events/minecraft:entity_spawned/randomize')), 2)
+        self.assertNotEqual(entity.get_jsonpath(path), None)
+        entity.delete_jsonpath(path)
+        self.assertEqual(entity.get_jsonpath(path), None)
+        self.assertEqual(len(entity.get_jsonpath('minecraft:entity/events/minecraft:entity_spawned/randomize')), 2)
+
         # Delete complex structure
         entity.delete_jsonpath('minecraft:entity')
         self.assertFalse(entity.jsonpath_exists('minecraft:entity'))
@@ -660,5 +731,39 @@ class TestJsonPathAccess(unittest.TestCase):
         with self.assertRaises(AssetNotFoundError):
             dolphin.get_jsonpath('dne')
 
+class TestFormatVersion(unittest.TestCase):
+    """
+    Tests format version getters, errors and comparison
+    """
+
+    def setUp(self) -> None:
+        self.project = Project('./content/bp/', './content/rp/')
+        self.bp = self.project.behavior_pack
+        self.rp = self.project.resource_pack
+        self.entity = self.bp.get_entity('minecraft:dolphin')
+        self.recipe = self.bp.get_recipe('minecraft:acacia_boat')
+        self.loot_table = self.bp.get_loot_table('loot_tables/entities/dolphin.json')
+
+    def test_format_version(self):
+        # Test getters
+        self.assertEqual(self.entity.format_version, '1.16.0')
+        self.assertEqual(self.recipe.format_version, '1.12.0')
+        with self.assertRaises(AttributeError):
+            self.loot_table.format_version
+
+        # Test Initaliser
+        self.assertEqual(FormatVersion('1'), '1.0.0.0')
+        self.assertEqual(FormatVersion('12.4'), FormatVersion(FormatVersion('12.4.0.3.4')))
+
+        # Test comparison
+        self.assertTrue(self.entity.format_version > self.recipe.format_version)
+
+        # Test setter
+        self.entity.format_version = '1.17.0'
+        self.assertEqual(self.entity.format_version, FormatVersion('1.17'))
+
+
+        
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
